use std::path::Path;

use anyhow::Result;
use chrono::IsoWeek;
use prettytable::format::FormatBuilder;
use prettytable::{cell, row, Cell, Table};
use smooth::{MultiSmooth, Smooth};

use crate::cli::Color;

pub fn run<P1, P2>(gleeo: P1, sqlite: P2, color: Color) -> Result<()>
where
    P1: AsRef<Path>,
    P2: AsRef<Path>,
{
    let debit = crate::debit::all(sqlite)?.weekly();
    let credit = crate::credit::all(gleeo)?.weekly();

    let mut weeks: Vec<IsoWeek> = vec![];
    weeks.extend(debit.keys());
    weeks.extend(credit.keys());
    weeks.sort_unstable();
    weeks.dedup();

    let mut credit_weekly = Vec::with_capacity(weeks.len());
    let mut debit_weekly = Vec::with_capacity(weeks.len());
    let mut balance_weekly = Vec::with_capacity(weeks.len());
    let mut rolling_weekly = Vec::with_capacity(weeks.len());

    let mut total_credit = f64::default();
    let mut total_debit = f64::default();
    let mut total_balance = f64::default();

    for week in &weeks {
        let credit = credit.get(week).copied().unwrap_or_default();

        // ALLOW time-tracked entries won't be too large
        #[allow(clippy::cast_precision_loss)]
        let credit = (credit as f64) / 3600.0;

        credit_weekly.push(credit);

        let debit = debit.get(week).copied().unwrap_or_default();
        debit_weekly.push(debit);

        let balance = credit - debit;
        balance_weekly.push(balance);

        total_credit += credit;
        total_debit += debit;
        total_balance += balance;

        rolling_weekly.push(total_balance);
    }

    credit_weekly.smooth_mut();
    balance_weekly.smooth_mut();
    rolling_weekly.smooth_mut();

    let mut table = Table::new();
    let format = FormatBuilder::new().column_separator(' ').build();
    table.set_format(format);
    table.set_titles(
        row![bu->"Week", bu->"Debit", bu->"Credit", bu->"Balance", bu->"Rolling"],
    );

    for (index, week) in weeks.iter().enumerate() {
        let mut row = row![
            format!("{:?}", week),
            r->debit_weekly[index],
            r->credit_weekly[index]
        ];

        row.add_cell(color_by_balance(balance_weekly[index]));
        row.add_cell(color_by_balance(rolling_weekly[index]));

        table.add_row(row);
    }

    if table.print_tty(color.with_atty()).is_ok() {
        println!();
        println!("total  debit: {} hours", total_debit.smooth());
        println!("total credit: {} hours", total_credit.smooth());
        println!();
        println!("balance: {}", total_balance.smooth());
        println!();
    }

    Ok(())
}

fn color_by_balance(a: f64) -> Cell {
    let cell = cell!(a);

    let style = if a.is_sign_positive() {
        "Fgr"
    } else if a.is_sign_negative() {
        "Frr"
    } else {
        "r"
    };

    cell.style_spec(style)
}
