use std::convert::TryFrom;
use std::path::{Path, PathBuf};

use anyhow::Result;
use atty::Stream;
use clap::{crate_description, crate_name, crate_version};
use clap::{App, AppSettings, Arg, ArgGroup, ArgMatches};

pub enum Mode {
    Balance,
    ListCredit,
    ListDebit,
}

#[derive(Clone, Copy, Debug)]
pub enum Color {
    Yes,
    No,
    Always,
}

impl Color {
    pub fn with_atty(self) -> bool {
        match self {
            Color::Yes => atty::is(Stream::Stdout),
            Color::No => false,
            Color::Always => true,
        }
    }
}

pub struct Arguments {
    pub mode: Mode,
    pub color: Color,
    pub gleeo: Option<PathBuf>,
    pub sqlite: Option<PathBuf>,
}

impl TryFrom<ArgMatches<'_>> for Arguments {
    type Error = anyhow::Error;

    fn try_from(args: ArgMatches) -> Result<Self, Self::Error> {
        let mode = if args.is_present("list-credit") {
            Mode::ListCredit
        } else if args.is_present("list-debit") {
            Mode::ListDebit
        } else {
            Mode::Balance
        };

        let color = match args.value_of("color").unwrap() {
            "yes" => Color::Yes,
            "always" => Color::Always,
            "no" => Color::No,
            _ => unreachable!("clap::Arg::possible_values"),
        };

        let gleeo = args.value_of("gleeo").map(|p| Path::new(p).into());
        let sqlite = args.value_of("sqlite").map(|p| Path::new(p).into());

        Ok(Self {
            mode,
            color,
            gleeo,
            sqlite,
        })
    }
}

pub fn args() -> Result<Arguments> {
    let colorize = atty::is(Stream::Stdout);

    let cli = build(colorize);
    let args = cli.get_matches();
    let arguments = Arguments::try_from(args)?;

    Ok(arguments)
}

pub fn build(colorize: bool) -> App<'static, 'static> {
    let colorize = if colorize {
        AppSettings::ColoredHelp
    } else {
        AppSettings::ColorNever
    };

    let list_credit = Arg::with_name("list-credit")
        .long("list-credit")
        .help("list credit");

    let list_debit = Arg::with_name("list-debit")
        .long("list-debit")
        .help("list debit");

    let mode = ArgGroup::with_name("mode")
        .arg("list-credit")
        .arg("list-debit");

    let color = Arg::with_name("color")
        .long("color")
        .help("control ANSI color escapes")
        .takes_value(true)
        .possible_values(&["yes", "no", "always"])
        .default_value("yes");

    let sqlite = Arg::with_name("sqlite")
        .long("debit")
        .takes_value(true)
        .help("sqlite debit file")
        .required_unless("list-credit");

    let gleeo = Arg::with_name("gleeo")
        .long("gleeo")
        .takes_value(true)
        .help("gleeo credit file")
        .required_unless("list-debit");

    App::new(crate_name!())
        .about(crate_description!())
        .version(crate_version!())
        .global_setting(colorize)
        .help_short("?")
        .help_message("show this help output")
        .version_message("show version")
        .arg(list_credit)
        .arg(list_debit)
        .group(mode)
        .arg(color)
        .arg(sqlite)
        .arg(gleeo)
}
