use std::path::Path;

use anyhow::{anyhow, Context, Result};
use chrono::{DateTime, Local};

use super::Entry;

pub fn from_path<P>(path: P) -> Result<Vec<Entry>>
where
    P: AsRef<Path>,
{
    let path = path.as_ref();

    let mut reader = csv::Reader::from_path(path)
        .with_context(|| format!("parsing {}", path.display()))?;

    let headers = reader
        .headers()
        .with_context(|| format!("parsing headers in {}", path.display()))?;

    let duration_index = headers
        .iter()
        .position(|h| h == "Decimal Duration")
        .ok_or_else(|| {
            anyhow!("no 'Decimal Duration' column in {}", path.display())
        })?;

    let start_index = headers
        .iter()
        .position(|h| h == "Start")
        .ok_or_else(|| anyhow!("no 'Start' column in {}", path.display()))?;

    let tz_index =
        headers
            .iter()
            .position(|h| h == "TimeZone")
            .ok_or_else(|| {
                anyhow!("no 'TimeZone' column in {}", path.display())
            })?;

    let mut credit = vec![];

    for record in reader.records().enumerate() {
        let (record_index, record) = record;

        // bump record index so it starts at 1 (readability)
        let record_index = record_index + 1;

        let record = record
            .with_context(|| format!("parsing record {}", record_index))?;

        let hours = record.get(duration_index).ok_or_else(|| {
            anyhow!("no 'Decimal Duration' field in record {}", record_index)
        })?;

        let hours = hours.parse::<f64>().with_context(|| {
            format!(
                "can't parse {:?} as f64 in record {}",
                hours, record_index
            )
        })?;

        // ALLOW time-tracked entries won't be too large
        #[allow(clippy::cast_possible_truncation)]
        let seconds = (hours * 3600.0) as i64;

        let start = record.get(start_index).ok_or_else(|| {
            anyhow!("no 'Start' field in record {}", record_index)
        })?;

        let tz = record.get(tz_index).ok_or_else(|| {
            anyhow!("no 'TimeZone' field in record {}", record_index)
        })?;

        let tz = match tz.strip_prefix("UTC") {
            Some(tz) => tz,
            None => tz,
        };

        let tz = if tz.len() == 5 {
            tz.replace("+", "+0")
        } else {
            tz.into()
        };

        let start = format!("{} {}", start, tz);

        let dt = DateTime::parse_from_str(&start, "%Y-%m-%d %H:%M %:z")
            .with_context(|| format!("parsing start date {}", start))?;

        let dt: DateTime<Local> = DateTime::from(dt);
        let date = dt.date();

        credit.push(Entry { date, seconds });
    }

    Ok(credit)
}

#[cfg(test)]
mod tests {
    #[test]
    fn decimal_duration() {
        let s = "0.3333333333333333";
        let h = s.parse::<f64>().unwrap();
        let seconds = (h * 3600.0) as i64;
        assert_eq!(seconds, 1200);
    }
}
